@@ -35,29 +35,21 @@ void on_about (GtkWidget* widget, void* data)
 	char * authors[] = {"Jean Bréfort", NULL};
 	char * artists[] = {"Nestor Diaz", NULL};
 	char * documentors[] = {NULL};
-	char license[] = "This program is free software; you can redistribute it and/or" 
-		"modify it under the terms of the GNU General Public License as"
- 		"published by the Free Software Foundation; either version 2 of the"
+	char license[] = "This program is free software; you can redistribute it and/or\n" 
+		"modify it under the terms of the GNU General Public License as\n"
+ 		"published by the Free Software Foundation; either version 2 of the\n"
 		"License, or (at your option) any later version.\n\n"
-		"This program is distributed in the hope that it will be useful,"
-		"but WITHOUT ANY WARRANTY; without even the implied warranty of"
-		"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the"
+		"This program is distributed in the hope that it will be useful,\n"
+		"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
+		"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
 		"GNU General Public License for more details.\n\n"
-		"You should have received a copy of the GNU General Public License"
-		"along with this program; if not, write to the Free Software"
-		"Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307"
+		"You should have received a copy of the GNU General Public License\n"
+		"along with this program; if not, write to the Free Software\n"
+		"Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307\n"
 		"USA";
 /* Note to translators: replace the following string with the appropriate credits for you lang */
 	char *translator_credits = _("translator_credits");
-/*	char * translators = g_strconcat (
-		_("Jean Bréfort <jean.brefort@ac-dijon.fr>: French"), "\n",
-		_("Christian Neumair <christian-neumair@web.de>: German"), "\n",
-		_("Costantino Ceoldo: Italian"), "\n",
-		_("Ben Luo: Chinese"), "\n",
-		_("Michał Sałaban: Polish"), "\n",
-		_("Nikodem Kuznik: Polish"),
-		NULL);*/
-	GdkPixbuf* pixbuf = gdk_pixbuf_new_from_file ( DATADIR"/pixmaps/gchempaint_logo.png", NULL);
+	GdkPixbuf* pixbuf = gdk_pixbuf_new_from_file ( DATADIR"/"PACKAGE"/pixmaps/gchempaint_logo.png", NULL);
 	gtk_show_about_dialog (NULL,
 					"name", "GChemPaint",
 					"authors", authors,
@@ -66,20 +58,12 @@ void on_about (GtkWidget* widget, void* data)
 					"copyright", _("(C) 2001-2005 by Jean Bréfort"),
 					"license", license,
 					"logo", pixbuf,
+					"icon-name", "gchempaint",
 					"translator_credits", strcmp (translator_credits, "translator_credits") != 0 ? 
 											(const char *)translator_credits : NULL,
 					"version", VERSION,
 					"website", "http://www.nongnu.org/gchempaint",
 					NULL);
-	/*gnome_about_new (_("GChemPaint"), VERSION,
-										_("(C) 2001-2005 by Jean Bréfort"),
-										_("GChemPaint is a 2D chemical structures editor for Gnome"),
-										(const gchar**) authors, (const gchar**) documentors,
-										strcmp (translator_credits, "translator_credits") != 0 ? 
-											(const char *)translator_credits : NULL,
-										pixbuf);*/
 	if (pixbuf != NULL)
 		g_object_unref (pixbuf);
-//	g_free (translators);
-//	gtk_widget_show_all (about);
 }
