@@ -70,7 +70,7 @@ public:
 	virtual bool SaveNode(xmlDocPtr xml, xmlNodePtr node, GtkTextIter *iter, GtkTextIter *end, GSList*& TagList, GSList*& OtherTagList);
 	virtual bool Load(xmlNodePtr);
 	bool LoadSelection(xmlNodePtr node, GtkTextIter *iter);
-	bool LoadNode(xmlNodePtr, GtkTextIter* CurIter);
+	bool LoadNode(xmlNodePtr, GtkTextIter* CurIter, int level = 0);
 	bool LoadTaggedNode(xmlNodePtr, GtkTextIter* CurIter, const char* TagName);
 	virtual void Add(GtkWidget* w);
 	virtual void Update(GtkWidget* w);
@@ -83,6 +83,7 @@ public:
 	bool OnEvent(GdkEvent *event);
 	void GetSize(double& x, double& y) {x = m_length; y = m_height;}
 	static void InitTags(const char* FontName);
+	virtual double GetYAlign ();
 };
 
 #endif	//GCHEMPAINT_TEXT_H
