@@ -4,9 +4,9 @@
  * GChemPaint library
  * view.cc
  *
- * Copyright (C) 2001-2004
+ * Copyright (C) 2001-2005
  *
- * Developed by Jean Bréfort <jean.brefort@ac-dijon.fr>
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -201,8 +201,10 @@ gcpView::gcpView(gcpDocument *pDoc, bool Embedded)
 	m_DefaultArrowHeadB = DefaultArrowHeadB;
 	m_DefaultArrowHeadC = DefaultArrowHeadC;
 	m_DefaultArrowDist = DefaultArrowDist;
-	m_DefaultArrowPadding = DefaultArrowPadding;
+	m_DefaultObjectPadding = DefaultObjectPadding;
 	m_DefaultSignPadding = DefaultSignPadding;
+	m_ArrowPadding = DefaultArrowPadding;
+	m_StoichiometryPadding = DefaultStoichiometryPadding;
 	m_width = 400;
 	m_height = 300;
 	m_ActiveRichText = NULL;
@@ -290,7 +292,7 @@ bool gcpView::OnEvent(GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget)
 			 	m_UIManager = gtk_ui_manager_new ();
 				result = pActiveTool->OnRightButtonClicked(this, Obj, event->button.x, event->button.y, m_UIManager);
 				if (Obj)
-					result |= Obj->BuildContextualMenu (m_UIManager);
+					result |= Obj->BuildContextualMenu (m_UIManager, Obj);
 				if (result) {
 					GtkWidget *w = gtk_ui_manager_get_widget (m_UIManager, "/popup");
 					gtk_menu_popup (GTK_MENU (w), NULL, NULL, NULL, NULL, 3,  gtk_get_current_event_time ());
@@ -355,7 +357,7 @@ GtkWidget* gcpView::CreateNewWidget()
 		m_pData->HashWidth = m_DefaultHashWidth * DefaultZoomFactor;
 		m_pData->HashDist = m_DefaultHashDist * DefaultZoomFactor;
 		m_pData->StereoBondWidth = m_DefaultStereoBondWidth * DefaultZoomFactor;
-		m_pData->ArrowPadding = m_DefaultArrowPadding * DefaultZoomFactor;
+		m_pData->ObjectPadding = m_DefaultObjectPadding * DefaultZoomFactor;
 		m_pData->SignPadding = m_DefaultSignPadding * DefaultZoomFactor;
 		m_pData->Group = GNOME_CANVAS_GROUP (gnome_canvas_item_new (
 									gnome_canvas_root(GNOME_CANVAS(m_pWidget)),
@@ -461,6 +463,14 @@ void gcpView::UpdateFont()
 	pango_layout_set_text(pl, "lj", 2);
 	pango_layout_get_extents(pl, &rect, NULL);
 	m_dFontHeight = rect.height / PANGO_SCALE;
+	g_object_unref(G_OBJECT(pl));
+	pl = pango_layout_new(m_PangoContext);
+	pango_layout_set_text(pl, "C", 1);
+	pango_layout_get_extents(pl, &rect, NULL);
+	PangoLayoutIter* iter = pango_layout_get_iter(pl);
+	double ascent = pango_layout_iter_get_baseline(iter) / PANGO_SCALE;
+	m_BaseLineOffset =  (double(rect.height / PANGO_SCALE) / 2.0) / DefaultZoomFactor;
+	g_object_unref(G_OBJECT(pl));
 }
 	
 void gcpView::Remove(Object* pObject)
@@ -487,29 +497,58 @@ void gcpView::UpdateLabel(const gchar* label)
 	if (GTK_IS_LABEL(l->data)) gtk_label_set_text((GtkLabel*)(l->data), label);
 }
 
-void gcpView::OnDeleteSelection(GtkWidget* w)
+void gcpView::OnDeleteSelection (GtkWidget* w)
 {
 	m_pWidget = w;
-	gcpApplication*	pApp = m_pDoc->GetApplication();
-	gcpTool* pActiveTool = pApp->GetActiveTool();
-	if (!pActiveTool->DeleteSelection())
+	gcpApplication*	pApp = m_pDoc->GetApplication ();
+	gcpTool* pActiveTool = pApp->GetActiveTool ();
+	if (!pActiveTool->DeleteSelection ())
 	{
-		m_pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(w), "data");
+		m_pData = (gcpWidgetData*) g_object_get_data (G_OBJECT(w), "data");
 		gcpWidgetData* pData;
 		std::list<GtkWidget*>::iterator j;
-		for (j = m_Widgets.begin(); j != m_Widgets.end(); j++)
-		{
-			if (*j == m_pWidget) continue;
-			pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(*j), "data");
-			pData->UnselectAll();
+		for (j = m_Widgets.begin (); j != m_Widgets.end (); j++) {
+			if (*j == m_pWidget)
+				continue;
+			pData = (gcpWidgetData*) g_object_get_data(G_OBJECT(*j), "data");
+			pData->UnselectAll ();
+		}
+		Object *pObject, *Group;
+		set<string> ModifiedObjects;
+		bool modify = false;
+		list<Object *>::iterator i, iend = m_pData->SelectedObjects.end ();
+		// first search if we are deleting top objects or not
+		for (i = m_pData->SelectedObjects.begin (); i != iend; i++)
+			if ((*i)->GetGroup ()) {
+				modify = true;
+				break;
+			}
+		gcpOperation *Op = m_pDoc->GetNewOperation (((modify)?
+								GCP_MODIFY_OPERATION: GCP_DELETE_OPERATION));
+		while (!m_pData->SelectedObjects.empty ()) {
+			pObject = m_pData->SelectedObjects.front ();
+			Group = pObject->GetGroup ();
+			if (Group && 
+				(ModifiedObjects.find (Group->GetId ()) == ModifiedObjects.end ())) {
+				Op->AddObject (Group);
+				ModifiedObjects.insert (Group->GetId ());
+			} else
+				Op->AddObject (pObject);
+			m_pData->SelectedObjects.front ()->Lock ();
+			m_pDoc->Remove (m_pData->SelectedObjects.front ());
+		}
+		m_pData->ClearSelection ();
+		set<string>::iterator k, kend = ModifiedObjects.end();
+		for (k = ModifiedObjects.begin(); k != kend; k++) {
+			pObject = m_pDoc->GetDescendant ((*k).c_str ());
+			if (pObject)
+				Op->AddObject (pObject, 1);
 		}
-		while (!m_pData->SelectedObjects.empty()) m_pDoc->Remove(m_pData->SelectedObjects.front());
-		m_pData->ClearSelection();
 	}
-	m_pDoc->FinishOperation();
-	pApp->ActivateMenu("Copy", false);
-	pApp->ActivateMenu("Cut", false);
-	pApp->ActivateMenu("Erase", false);
+	m_pDoc->FinishOperation ();
+	pApp->ActivateMenu ("Copy", false);
+	pApp->ActivateMenu ("Cut", false);
+	pApp->ActivateMenu ("Erase", false);
 }
 
 GtkTargetEntry const targets[] = {
@@ -775,7 +814,7 @@ void gcpView::ExportImage (const char* filename, const char* type)
 	ArtDRect rect;
 	m_pData->GetObjectBounds (m_pDoc, &rect);
 	m_pData->ShowSelection (false);
-	int w = (int) (ceil (rect.x1) - floor (rect.x0)), h = (int) (ceil (rect.y1) - floor (rect.y0));
+	int w = (int) (ceil (rect.x1) - floor (rect.x0)) + 1, h = (int) (ceil (rect.y1) - floor (rect.y0)) + 1;
 	if (!strcmp (type, "eps")) {
 		GnomePrintConfig* config = gnome_print_config_default();
 		GnomePrintContext *pc;
@@ -799,7 +838,7 @@ void gcpView::ExportImage (const char* filename, const char* type)
 		char *command_line = g_strconcat ("ps2eps ", tmpname, NULL);
 		g_spawn_command_line_sync (command_line, NULL, NULL, NULL, NULL);
 		g_free (command_line);
-		command_line = g_strconcat ("mv -f ", tmpname, ".eps ", filename);
+		command_line = g_strconcat ("mv -f ", tmpname, ".eps ", filename, NULL);
 		g_spawn_command_line_async (command_line, NULL);
 		remove (tmpname);
 		g_free (tmpname);
@@ -854,7 +893,6 @@ void gcpView::ExportImage (const char* filename, const char* type)
 		xmlFreeDoc (doc);
 	} else {
 		gnome_canvas_update_now (GNOME_CANVAS (m_pWidget));
-		GdkDrawable *drawable = GDK_DRAWABLE (gtk_widget_get_parent_window (m_pWidget));
 		GdkPixbuf *pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, w, h);
 		GnomeCanvasBuf buf;
 		buf.buf = gdk_pixbuf_get_pixels (pixbuf);
@@ -864,9 +902,19 @@ void gcpView::ExportImage (const char* filename, const char* type)
 		buf.rect.y1 = (int) ceil (rect.y1);
 		buf.buf_rowstride = gdk_pixbuf_get_rowstride (pixbuf);
 		buf.bg_color = 0xffffff;
+		buf.is_buf = 1;
 		(* GNOME_CANVAS_ITEM_GET_CLASS (m_pData->Group)->render) (GNOME_CANVAS_ITEM (m_pData->Group), &buf);
 		gdk_pixbuf_save (pixbuf, filename, type, NULL, NULL);
 		g_object_unref (pixbuf);
 	}
 	m_pData->ShowSelection (true);
 }
+
+void gcpView::EnsureSize ()
+{
+	GnomeCanvas *canvas = GNOME_CANVAS (m_pWidget);
+	while (canvas->idle_id)
+		gtk_main_iteration();
+	gnome_canvas_update_now (canvas);
+	g_signal_emit_by_name (m_pWidget, "update_bounds");
+}
