@@ -34,6 +34,10 @@
  *          Raph Levien <raph@gimp.org>
  */
 
+#include "gchempaint-config.h"
+#include <math.h>
+#include <string.h>
+
 #include <libgnomecanvas/gnome-canvas.h>
 #include <libgnomecanvas/gnome-canvas-util.h>
 
