@@ -34,7 +34,7 @@
  * Port to Pango co-done by Gergõ Érdi <cactus@cactus.rulez.org>
  */
 
-#include "config.h"
+#include "gchempaint-config.h"
 #include <math.h>
 #include "gcp-canvas-text.h"
 #include "gprintable.h"
@@ -104,7 +104,6 @@ void gnome_canvas_text_ext_print (GPrintable *printable, GnomePrintContext *pc)
 	double x , y;
 	g_return_if_fail (GNOME_IS_CANVAS_TEXT_EXT (printable));
 	text = GNOME_CANVAS_TEXT (printable);
-	text_ext = GNOME_CANVAS_TEXT_EXT (printable);
 	if (!text_ext->rendered)
 		gnome_canvas_text_ext_ensure_rendered (text_ext);
 	gnome_print_gsave (pc);
@@ -169,6 +168,8 @@ gnome_canvas_text_ext_export_svg (GPrintable *printable, xmlDocPtr doc, xmlNodeP
 {
 	GnomeCanvasText *text;
 	double x , y;
+	xmlNodePtr child;
+	char *buf;
 	GnomeCanvasTextExt *text_ext;
 
 	g_return_if_fail (GNOME_IS_CANVAS_TEXT_EXT (printable));
@@ -228,12 +229,10 @@ static void
 gnome_canvas_text_ext_ensure_rendered (GnomeCanvasTextExt *text)
 {
 	double x1, y1, x2, y2;
-	int w, h;
-	GdkPixbuf *pixbuf;
-	GnomeCanvasBuf buf;
 	gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (text), &x1, &y1, &x2, &y2);
-	w = (int) (ceil (x2) - floor (x1)) + 1, h = (int) (ceil (y2) - floor (y1)) + 1;
-	pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, w, h);
+	int w = (int) (ceil (x2) - floor (x1)) + 1, h = (int) (ceil (y2) - floor (y1)) + 1;
+	GdkPixbuf *pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, w, h);
+	GnomeCanvasBuf buf;
 	buf.buf = gdk_pixbuf_get_pixels (pixbuf);
 	buf.rect.x0 = (int) floor (x1);
 	buf.rect.x1 = (int) ceil (x2);
