@@ -4,9 +4,9 @@
  * GChemPaint atoms plugin
  * elementtool.cc 
  *
- * Copyright (C) 2001-2004
+ * Copyright (C) 2001-2005
  *
- * Developed by Jean Bréfort <jean.brefort@ac-dijon.fr>
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -26,7 +26,7 @@
 
 #include "gchempaint-config.h"
 #include "elementtool.h"
-#include <chemistry/element.h>
+#include <gcu/element.h>
 #include "lib/document.h"
 #include "lib/settings.h"
 #include "lib/application.h"
@@ -103,63 +103,56 @@ void gcpElementTool::OnDrag()
 		}
 }
 
-void gcpElementTool::OnRelease()
+void gcpElementTool::OnRelease ()
 {
-	int CurZ = m_pApp->GetCurZ() ;
+	int CurZ = m_pApp->GetCurZ () ;
 	if (m_bChanged)
 	{
+		gcpDocument* pDoc = m_pView->GetDoc ();
 		if (m_pObject)
 		{
-			gcpMolecule* pMol = (gcpMolecule*)m_pObject->GetMolecule();
-			gcpDocument* pDoc = m_pView->GetDoc();
-			gcpOperation* pOp = pDoc-> GetNewOperation(GCP_MODIFY_OPERATION);
-			pOp->AddObject(pMol, 0);
-			Object* parent = m_pObject->GetParent();
-			if (m_nState & GDK_CONTROL_MASK && (parent->GetType() == FragmentType))
-			{//If m_pObject points to an atom inside a fragment, replace the whole fragment.
-				gcpAtom* pAtom = ((gcpFragment*)parent)->GetAtom();
+			gcpMolecule* pMol = (gcpMolecule*) m_pObject->GetMolecule ();
+			gcpOperation* pOp = pDoc-> GetNewOperation (GCP_MODIFY_OPERATION);
+			Object *pObj = m_pObject->GetGroup ();
+			pOp->AddObject (pObj, 0);
+			Object* parent = m_pObject->GetParent ();
+			if (m_nState & GDK_CONTROL_MASK && (parent->GetType () == FragmentType)) {
+				//If m_pObject points to an atom inside a fragment, replace the whole fragment.
+				gcpAtom* pAtom = ((gcpFragment*) parent)->GetAtom ();
 				map<Atom*, Bond*>::iterator i;
-				gcpBond *pBond = (gcpBond*)pAtom->GetFirstBond(i);
+				gcpBond *pBond = (gcpBond*) pAtom->GetFirstBond (i);
 				double x, y;
-				pAtom->GetCoords(&x, &y);
-				gcpAtom* pNewAtom = new gcpAtom(CurZ, x, y, 0.);
-				pMol->Remove(parent);
-				m_pView->Remove(parent);
-				m_pView->AddObject(pNewAtom);
-				parent->SetParent(NULL);
-				pMol->AddAtom(pNewAtom);
-				pNewAtom->SetId((gchar*)pAtom->GetId());
-				if (pBond)
-				{
-					pBond->ReplaceAtom(pAtom, pNewAtom);
-					pNewAtom->AddBond(pBond);
+				pAtom->GetCoords (&x, &y);
+				gcpAtom* pNewAtom = new gcpAtom (CurZ, x, y, 0.);
+				pMol->Remove (parent);
+				m_pView->Remove (parent);
+				m_pView->AddObject (pNewAtom);
+				parent->SetParent (NULL);
+				pMol->AddAtom (pNewAtom);
+				pNewAtom->SetId ((gchar*) pAtom->GetId ());
+				if (pBond) {
+					pBond->ReplaceAtom (pAtom, pNewAtom);
+					pNewAtom->AddBond (pBond);
 				}
-				pNewAtom->Update();
-				m_pView->Update(pNewAtom);
+				pNewAtom->Update ();
+				m_pView->Update (pNewAtom);
 				delete parent;
-			}
-			else
-			{
-				((gcpAtom*)m_pObject)->SetZ(CurZ);
-				m_pView->Update((gcpAtom*)m_pObject);
+			} else {
+				((gcpAtom*) m_pObject)->SetZ (CurZ);
+				m_pView->Update ((gcpAtom*) m_pObject);
 			}	
-			pOp->AddObject(pMol, 1);
-			pDoc->FinishOperation();
-		}
-		else
-		{
-			gcpAtom* pAtom = new gcpAtom(CurZ, m_x0 / m_dZoomFactor, m_y0 / m_dZoomFactor, 0);
-			m_pView->GetDoc()->AddAtom(pAtom);
+			pOp->AddObject (pObj, 1);
+		} else {
+			gcpAtom* pAtom = new gcpAtom (CurZ, m_x0 / m_dZoomFactor, m_y0 / m_dZoomFactor, 0);
+			gcpOperation* pOp = pDoc-> GetNewOperation (GCP_ADD_OPERATION);
+			pDoc->AddAtom (pAtom);
+			pOp->AddObject (pAtom);
 		}
+		pDoc->FinishOperation ();
 	}
 }
 
 void gcpElementTool::SetOptions()
 {
-	gcpMendeleievDlg* dlg = (gcpMendeleievDlg*) m_pApp->GetDialog("Mendeleiev");
-	if (!dlg)
-	{
-		dlg = new gcpMendeleievDlg (m_pApp, m_pApp->GetCurZ());
-	}
-	else gdk_window_raise (GTK_WIDGET(dlg->GetWindow())->window); 
+	m_pApp->OnShowElements ();
 }
