@@ -4,9 +4,9 @@
  * GChemPaint
  * standaloneapp.cc
  *
- * Copyright (C) 2004
+ * Copyright (C) 2004-2005
  *
- * Developed by Jean Bréfort <jean.brefort@ac-dijon.fr>
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -72,6 +72,11 @@ static void on_file_save(GtkWidget* widget, gcpStandaloneApp* App)
 	App->OnSave();
 }
 
+static void on_file_save_as_image(GtkWidget* widget, gcpStandaloneApp* App)
+{
+	App->OnSaveAsImage ();
+}
+
 static void on_file_close(GtkWidget* widget, gcpStandaloneApp* App)
 {
 	App->OnFileClose ();
@@ -237,15 +242,17 @@ static GtkActionEntry entries[] = {
   { "FileMenu", NULL, N_("_File") },
 	  { "New", GTK_STOCK_NEW, N_("_New File"), NULL,
 		  N_("Create a new file"), G_CALLBACK (on_file_new) },
-	  { "Open", GTK_STOCK_OPEN, N_("_Open"), "<control>O",
+	  { "Open", GTK_STOCK_OPEN, N_("_Open..."), "<control>O",
 		  N_("Open a file"), G_CALLBACK (on_file_open) },
 	  { "Save", GTK_STOCK_SAVE, N_("_Save"), "<control>S",
 		  N_("Save the current file"), G_CALLBACK (on_file_save) },
-	  { "SaveAs", GTK_STOCK_SAVE, N_("Save _As"), "<shift><control>S",
-		  N_("Save the current file with a different name "), G_CALLBACK (on_file_save_as) },
-	  { "Print", GTK_STOCK_PRINT, N_("_Print"), "<control>P",
+	  { "SaveAs", GTK_STOCK_SAVE_AS, N_("Save _As..."), "<shift><control>S",
+		  N_("Save the current file with a different name"), G_CALLBACK (on_file_save_as) },
+	  { "SaveAsImage", GTK_STOCK_SAVE_AS, N_("Save As _Image..."), "<control>I",
+		  N_("Save the current file as an image"), G_CALLBACK (on_file_save_as_image) },
+	  { "Print", GTK_STOCK_PRINT, N_("_Print..."), "<control>P",
 		  N_("Print the current file"), G_CALLBACK (on_print) },
-	  { "Properties", GTK_STOCK_PROPERTIES, N_("Prope_rties"), NULL,
+	  { "Properties", GTK_STOCK_PROPERTIES, N_("Prope_rties..."), NULL,
 		  N_("Modify the file's properties"), G_CALLBACK (on_properties) },
 	  { "Close", GTK_STOCK_CLOSE, N_("_Close"), "<control>W",
 		  N_("Close the current file"), G_CALLBACK (on_file_close) },
@@ -253,7 +260,7 @@ static GtkActionEntry entries[] = {
 		  N_("Quit GChemPaint"), G_CALLBACK (on_quit) },
   { "EditMenu", NULL, N_("_Edit") },
 	  { "Undo", GTK_STOCK_UNDO, N_("_Undo"), "<control>Z",
-		  N_("Undo the lase action"), G_CALLBACK (on_undo) },
+		  N_("Undo the last action"), G_CALLBACK (on_undo) },
 	  { "Redo", GTK_STOCK_REDO, N_("_Redo"), "<shift><control>Z",
 		  N_("Redo the undone action"), G_CALLBACK (on_redo) },
 	  { "Cut", GTK_STOCK_CUT, N_("Cu_t"), "<control>X",
@@ -300,6 +307,7 @@ static const char *ui_description =
 "      <menuitem action='Open'/>"
 "      <menuitem action='Save'/>"
 "      <menuitem action='SaveAs'/>"
+"      <menuitem action='SaveAsImage'/>"
 "      <separator name='file-sep1'/>"
 "      <menuitem action='Print'/>"
 "      <separator name='file-sep2'/>"
@@ -391,12 +399,13 @@ gcpStandaloneApp::gcpStandaloneApp(): gcpApplication()
 	m_MessageId = 0;
 	m_Window = GTK_WINDOW (gtk_window_new (GTK_WINDOW_TOPLEVEL));
 	gtk_window_set_title (m_Window, _("GChemPaint"));
-	gtk_window_set_icon_from_file (m_Window, PIXMAPSDIR"/gchempaint.png", NULL);
+	gtk_window_set_icon_name (m_Window, "gchempaint");
 	g_signal_connect(G_OBJECT(m_Window), "destroy", G_CALLBACK(on_quit), this);
 	g_signal_connect(G_OBJECT(m_Window), "focus_in_event", G_CALLBACK(on_focus), this);
 	vbox = gtk_vbox_new (FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (m_Window), vbox);
 	action_group = gtk_action_group_new ("MenuActions");
+	gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
 	gtk_action_group_add_actions (action_group, entries, G_N_ELEMENTS (entries), this);
 	gtk_action_group_add_toggle_actions (action_group, toggle_entries, G_N_ELEMENTS (toggle_entries), this);
 
@@ -406,12 +415,10 @@ gcpStandaloneApp::gcpStandaloneApp(): gcpApplication()
 		"signal::disconnect_proxy", G_CALLBACK (on_disconnect_proxy), this,
 		NULL);
 	gtk_ui_manager_insert_action_group (m_UIManager, action_group, 0);
-	if (MailAgent[0])
 
 	accel_group = gtk_ui_manager_get_accel_group (m_UIManager);
 	gtk_window_add_accel_group (GTK_WINDOW (m_Window), accel_group);
 	
-	if (MailAgent[0])
 	error = NULL;
 	if (!gtk_ui_manager_add_ui_from_string (m_UIManager, ui_description, -1, &error))
 	  {
@@ -444,19 +451,22 @@ gcpStandaloneApp::gcpStandaloneApp(): gcpApplication()
 	m_statusId = gtk_statusbar_get_context_id (GTK_STATUSBAR (m_Bar), "status");
 	gtk_statusbar_push (GTK_STATUSBAR (m_Bar), m_statusId, _("Ready"));
 	gtk_box_pack_start (GTK_BOX (vbox), m_Bar, false, false, 0);
-	gtk_widget_show_all (GTK_WIDGET (m_Window));
 	SetMenu("Undo", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Undo"));
 	SetMenu("Redo", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Redo"));
 	SetMenu("Copy", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Copy"));
 	SetMenu("Cut", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Cut"));
 	SetMenu("Paste", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Paste"));
 	SetMenu("Erase", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Delete"));
+	SetMenu("Image", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/FileMenu/SaveAsImage"));
+	SetMenu("Save", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/FileMenu/Save"));
+	SetToolItem("Save", gtk_ui_manager_get_widget (m_UIManager, "/MainToolbar/Save"));
 	ActivateMenu("Undo", false);
 	ActivateMenu("Redo", false);
 	ActivateMenu("Copy", false);
 	ActivateMenu("Cut", false);
 	ActivateMenu("Paste", false);
 	ActivateMenu("Erase", false);
+	ActivateMenu("Image", false);
 
 	Callbacks["tools"] = (GCallback) on_tool_clicked;
 
@@ -516,7 +526,7 @@ void gcpStandaloneApp::OnFileNew()
 {
 	gchar tmp[32];
 	if (m_pActiveDoc && !m_pActiveDoc->GetView()->PrepareUnselect()) return;
-	g_snprintf(tmp, sizeof(tmp), _("Untitled%d"), m_NumWindow++);
+	g_snprintf(tmp, sizeof(tmp), _("Untitled %d"), m_NumWindow++);
 	GtkWidget* w = CreateView();
 	if (w)
 	{
@@ -530,20 +540,20 @@ void gcpStandaloneApp::OnFileNew()
 		gchar* buf = g_strconcat ("<ui><menubar name='MainMenu'><menu action='WindowsMenu'><menuitem action='",
 		tmp, "'/></menu></menubar></ui>", NULL);
 		error = NULL;
-		int id = gtk_ui_manager_add_ui_from_string (m_UIManager, buf, -1, NULL);
+		long id = gtk_ui_manager_add_ui_from_string (m_UIManager, buf, -1, NULL);
 		g_free (buf);
 		buf = g_strconcat ("/MainMenu/WindowsMenu/", tmp, NULL);
 		GtkWidget* item = gtk_ui_manager_get_widget (m_UIManager, buf);
 		g_free (buf);
 		g_signal_connect(G_OBJECT(item), "activate", (GtkSignalFunc)on_select, w); 
-		g_signal_connect(G_OBJECT(w), "expose_event", (GtkSignalFunc)on_page_select, NULL); 
+		g_signal_connect(G_OBJECT(w), "expose_event", (GtkSignalFunc)on_page_select, NULL);
 		g_object_set_data(G_OBJECT(w), "menu-id", (void*) id);
 		g_object_set_data(G_OBJECT(w), "label", label);
 		GtkScrolledWindow* scroll = (GtkScrolledWindow*)gtk_scrolled_window_new(NULL, NULL);
 		gtk_scrolled_window_set_policy(scroll, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 		gtk_scrolled_window_set_shadow_type (scroll, GTK_SHADOW_IN);
 		gtk_scrolled_window_add_with_viewport(scroll, w);
-		gtk_widget_set_size_request(GTK_WIDGET(scroll), 400, 300);
+		gtk_widget_set_size_request(GTK_WIDGET(scroll), 408, 308);
 		gtk_widget_show(GTK_WIDGET(scroll));
 		gtk_notebook_append_page(m_Book, GTK_WIDGET(scroll), label);
 		gtk_notebook_set_current_page(m_Book, g_list_length(m_Book->children) - 1);
@@ -579,7 +589,7 @@ void  gcpStandaloneApp::AddDocument(gcpDocument* pDoc)
 		gchar* buf = g_strconcat ("<ui><menubar name='MainMenu'><menu action='WindowsMenu'><menuitem action='",
 		winname, "'/></menu></menubar></ui>", NULL);
 		error = NULL;
-		int id = gtk_ui_manager_add_ui_from_string (m_UIManager, buf, -1, NULL);
+		long id = gtk_ui_manager_add_ui_from_string (m_UIManager, buf, -1, NULL);
 		g_free (buf);
 		buf = g_strconcat ("/MainMenu/WindowsMenu/", winname, NULL);
 		GtkWidget* item = gtk_ui_manager_get_widget (m_UIManager, buf);
@@ -592,7 +602,7 @@ void  gcpStandaloneApp::AddDocument(gcpDocument* pDoc)
 		gtk_scrolled_window_set_policy(scroll, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 		gtk_scrolled_window_set_shadow_type (scroll, GTK_SHADOW_IN);
 		gtk_scrolled_window_add_with_viewport(scroll, w);
-		gtk_widget_set_size_request(GTK_WIDGET(scroll), 400, 300);
+		gtk_widget_set_size_request(GTK_WIDGET(scroll), 408, 308);
 		gtk_widget_show(GTK_WIDGET(scroll));
 		gtk_notebook_append_page(m_Book, GTK_WIDGET(scroll), label);
 		gtk_notebook_set_current_page(m_Book, g_list_length(m_Book->children) - 1);
@@ -603,7 +613,7 @@ void  gcpStandaloneApp::AddDocument(gcpDocument* pDoc)
 void gcpStandaloneApp::RemoveDocument(gcpDocument* pDoc)
 {
 	GtkWidget *w = pDoc->GetWidget ();
-	gtk_ui_manager_remove_ui (m_UIManager, (int) g_object_get_data (G_OBJECT (w), "menu-id"));
+	gtk_ui_manager_remove_ui (m_UIManager, (long) g_object_get_data (G_OBJECT (w), "menu-id"));
 	m_Docs.remove (pDoc);
 }
 
@@ -838,6 +848,7 @@ void gcpStandaloneApp::OnPrint()
 	int do_preview, copies = 1, collate = 0;
 	GnomePrintDialog *gpd;
 	gpd = GNOME_PRINT_DIALOG (gnome_print_dialog_new(gpj, (const guchar*)_("Print"), GNOME_PRINT_DIALOG_COPIES));
+	gtk_window_set_icon_name (GTK_WINDOW (gpd), "gchempaint");
 	gnome_print_dialog_set_copies(gpd, copies, collate);
 	switch (gtk_dialog_run(GTK_DIALOG(gpd)))
 	{
@@ -862,7 +873,9 @@ void gcpStandaloneApp::OnPrint()
 	gnome_print_job_close(gpj);
 	if (do_preview)
 	{
-		gtk_widget_show (gnome_print_job_preview_new (gpj, (const guchar*)_("Preview")));
+		GtkWidget *preview = gnome_print_job_preview_new (gpj, (const guchar*)_("Preview"));
+		gtk_window_set_icon_name (GTK_WINDOW (preview), "gchempaint");
+		gtk_widget_show (preview);
 	}
 	else
 	{
@@ -1004,3 +1017,8 @@ void gcpStandaloneApp::ToggleMenu(const string& menuname, bool active)
 	if (gtk_check_menu_item_get_active ((GtkCheckMenuItem*) w) != active)
 		((GtkCheckMenuItem*) w)->active = active;
 }
+
+void gcpStandaloneApp::Show ()
+{
+	gtk_widget_show_all (GTK_WIDGET (m_Window));
+}
